/*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */

.service-grid-table {
  width: 0;
  white-space: nowrap;

  td {
    color: @gray-light;
    padding: 0.2em;
    text-align: center;
    width: 1em;
  }

  .rotate-45 {
    height: 8em;

    div {
      .transform(translate(0.4em, 2.8em) rotate(315deg));
      width: 1.5em;
    }
  }

  .service-grid-table-more {
    text-align: center;
    a {
      display: inline;
    }
  }
}

.joystick-pagination {
  margin: 0 auto;
  font-size: 130%;

  a {
    color: @text-color;
    outline: none;

    &:hover {
      color: @text-color-light;
    }
    &:focus, &:active {
      color: @icinga-blue;
    }
  }

  i {
    display: block;
    height: 1.5em;
    width: 1.5em;
  }
}

.service-grid-link {
  .bg-stateful();
  .rounded-corners();

  display: inline-block;
  height: 1.5em;
  vertical-align: middle;
  width: 1.5em;
}

form.filter-toggle {
  label:not(.toggle-switch) {
    display: inline-block;
    vertical-align: top;
    margin-left: .5em;
    color: @gray-light;
  }

  input[type="checkbox"]:checked ~ label {
    color: inherit;
  }
}
