<div class="controls">
    <?php if (! $this->compact): ?>
        <?= $this->tabs; ?>
    <?php endif ?>

    <div data-base-target='_next'>
        <?= $this->render('partials/comment/comment-header.phtml'); ?>
    </div>
</div>
<div class="content">

<h2><?= $this->translate('Comment detail information') ?></h2>
<table class="name-value-table">
  <tbody>
    <tr>
      <?php if ($this->comment->objecttype === 'service'): ?>
        <th> <?= $this->translate('Service') ?> </th>
        <td>
          <?= $this->icon('service', $this->translate('Service')); ?>
          <?= $this->link()->service(
                  $this->comment->service_description,
                  $this->comment->service_display_name,
                  $this->comment->host_name,
                  $this->comment->host_display_name
              );
          ?>
        </td>
     <?php else: ?>
        <th> <?= $this->translate('Host') ?> </th>
        <td>
          <?= $this->icon('host', $this->translate('Host')); ?>
          <?= $this->link()->host(
                  $this->comment->host_name,
                  $this->comment->host_display_name
              );
          ?>
        </td>
     <?php endif ?>
    </tr>

    <tr>
      <th><?= $this->translate('Author') ?></th>
      <td><?= $this->icon('user', $this->translate('User')) ?> <?= $this->escape($this->comment->author) ?></td>
    </tr>

    <tr>
      <th><?= $this->translate('Persistent') ?></th>
      <td><?= $this->escape($this->comment->persistent) ? $this->translate('Yes') : $this->translate('No') ?></td>
    </tr>

    <tr>
      <th><?= $this->translate('Created') ?></th>
      <td><?= $this->formatDateTime($this->comment->timestamp) ?></td>
    </tr>

    <tr>
      <th><?= $this->translate('Expires') ?></th>
      <td>
          <?= $this->comment->expiration ? sprintf(
                  $this->translate('This comment expires on %s at %s.'),
                  $this->formatDate($this->comment->expiration),
                  $this->formatTime($this->comment->expiration)
              ) : $this->translate('This comment does not expire.');
          ?>
      </td>
    </tr>

    <tr>
      <th><?= $this->translate('Comment') ?></th>
      <td><?= $this->nl2br($this->createTicketLinks($this->markdown($comment->comment))) ?></td>
    </tr>

    <?php if (isset($delCommentForm)): // Form is unset if the current user lacks the respective permission ?>
    <tr class="newsection">
      <th><?= $this->translate('Commands') ?></th>
      <td>
        <?= $delCommentForm ?>
      </td>
    </tr>
    <?php endif ?>

  </tbody>
</table>

</div>

