<?php
/* Icinga Web 2 | (c) 2013 Icinga Development Team | GPLv2+ */

// TODO: obsolete, remove once MonitoringBackend is in use everywhere

namespace Icinga\Module\Monitoring;

use Icinga\Module\Monitoring\Backend\MonitoringBackend;

class Backend extends MonitoringBackend
{
}
