<div class="controls">
<?= $this->tabs ?>
</div>
<table class="avp">
<tr><th><?= $this->translate('UI Debug') ?></th><td><a href="javascript:void(0);" onclick="icinga.ui.toggleDebug();"><?= $this->translate('toggle') ?></td></tr>
</table>
