<?php
$helpMessage = $this->translate(
    'Press and hold the Ctrl key while clicking on rows to select multiple rows or press and hold the Shift key to'
    .' select a range of rows',
    'Multi-selection help'
);
?>
<div class="selection-info" title="<?= $this->escape($helpMessage) ?>">
    <?= sprintf(
        /// TRANSLATORS: Please leave %s as it is because the selection counter is wrapped in a span tag for updating
        /// the counter via JavaScript
        $this->translate('%s row(s) selected', 'Multi-selection count'),
        '<span class="selection-info-count">0</span>'
    ) ?>
</div>
