<?php

use Icinga\Application\Icinga;
use Icinga\Web\StyleSheet;


$moduleName = $this->layout()->moduleName;
if ($moduleName !== 'default') {
    $moduleClass = ' icinga-module module-' . $moduleName;
} else {
    $moduleClass = '';
}

$logoPath = Icinga::app()->getBootstrapDirectory() . '/img/icinga-logo-big-dark.png';
$logo = base64_encode(file_get_contents($logoPath));


?><!DOCTYPE html>
<html>
<head>
<style>
<?= StyleSheet::forPdf() ?>
</style>
<base href="<?= $this->serverUrl() ?>">
</head>
<body>
<div id="header">
  <table>
    <tbody>
      <tr>
        <th><?= $this->escape($this->title) ?></th>
        <td style="text-align: right;"><img width="75" src="data:image/png;base64,<?= $logo ?>"></td>
      </tr>
    </tbody>
  </table>
</div>
<div id="footer">
  <div class="page-number"></div>
</div>
<div class="<?= $moduleClass ?>">
    <?= $this->render('inline.phtml') ?>
</div>
</body>
</html>
