
.icon-dashboard:before { content: '\e800'; } /* '' */
.icon-user:before { content: '\e801'; } /* '' */
.icon-users:before { content: '\e802'; } /* '' */
.icon-ok:before { content: '\e803'; } /* '' */
.icon-cancel:before { content: '\e804'; } /* '' */
.icon-plus:before { content: '\e805'; } /* '' */
.icon-minus:before { content: '\e806'; } /* '' */
.icon-folder-empty:before { content: '\e807'; } /* '' */
.icon-download:before { content: '\e808'; } /* '' */
.icon-upload:before { content: '\e809'; } /* '' */
.icon-git:before { content: '\e80a'; } /* '' */
.icon-cubes:before { content: '\e80b'; } /* '' */
.icon-database:before { content: '\e80c'; } /* '' */
.icon-gauge:before { content: '\e80d'; } /* '' */
.icon-sitemap:before { content: '\e80e'; } /* '' */
.icon-sort-name-up:before { content: '\e80f'; } /* '' */
.icon-sort-name-down:before { content: '\e810'; } /* '' */
.icon-megaphone:before { content: '\e811'; } /* '' */
.icon-bug:before { content: '\e812'; } /* '' */
.icon-tasks:before { content: '\e813'; } /* '' */
.icon-filter:before { content: '\e814'; } /* '' */
.icon-off:before { content: '\e815'; } /* '' */
.icon-book:before { content: '\e816'; } /* '' */
.icon-paste:before { content: '\e817'; } /* '' */
.icon-scissors:before { content: '\e818'; } /* '' */
.icon-globe:before { content: '\e819'; } /* '' */
.icon-cloud:before { content: '\e81a'; } /* '' */
.icon-flash:before { content: '\e81b'; } /* '' */
.icon-barchart:before { content: '\e81c'; } /* '' */
.icon-down-dir:before { content: '\e81d'; } /* '' */
.icon-up-dir:before { content: '\e81e'; } /* '' */
.icon-left-dir:before { content: '\e81f'; } /* '' */
.icon-right-dir:before { content: '\e820'; } /* '' */
.icon-down-open:before { content: '\e821'; } /* '' */
.icon-right-open:before { content: '\e822'; } /* '' */
.icon-up-open:before { content: '\e823'; } /* '' */
.icon-left-open:before { content: '\e824'; } /* '' */
.icon-up-big:before { content: '\e825'; } /* '' */
.icon-right-big:before { content: '\e826'; } /* '' */
.icon-left-big:before { content: '\e827'; } /* '' */
.icon-down-big:before { content: '\e828'; } /* '' */
.icon-resize-full-alt:before { content: '\e829'; } /* '' */
.icon-resize-full:before { content: '\e82a'; } /* '' */
.icon-resize-small:before { content: '\e82b'; } /* '' */
.icon-move:before { content: '\e82c'; } /* '' */
.icon-resize-horizontal:before { content: '\e82d'; } /* '' */
.icon-resize-vertical:before { content: '\e82e'; } /* '' */
.icon-zoom-in:before { content: '\e82f'; } /* '' */
.icon-block:before { content: '\e830'; } /* '' */
.icon-zoom-out:before { content: '\e831'; } /* '' */
.icon-lightbulb:before { content: '\e832'; } /* '' */
.icon-clock:before { content: '\e833'; } /* '' */
.icon-volume-up:before { content: '\e834'; } /* '' */
.icon-volume-down:before { content: '\e835'; } /* '' */
.icon-volume-off:before { content: '\e836'; } /* '' */
.icon-mute:before { content: '\e837'; } /* '' */
.icon-mic:before { content: '\e838'; } /* '' */
.icon-endtime:before { content: '\e839'; } /* '' */
.icon-starttime:before { content: '\e83a'; } /* '' */
.icon-calendar-empty:before { content: '\e83b'; } /* '' */
.icon-calendar:before { content: '\e83c'; } /* '' */
.icon-wrench:before { content: '\e83d'; } /* '' */
.icon-sliders:before { content: '\e83e'; } /* '' */
.icon-services:before { content: '\e83f'; } /* '' */
.icon-service:before { content: '\e840'; } /* '' */
.icon-phone:before { content: '\e841'; } /* '' */
.icon-file-pdf:before { content: '\e842'; } /* '' */
.icon-file-word:before { content: '\e843'; } /* '' */
.icon-file-excel:before { content: '\e844'; } /* '' */
.icon-doc-text:before { content: '\e845'; } /* '' */
.icon-trash:before { content: '\e846'; } /* '' */
.icon-comment-empty:before { content: '\e847'; } /* '' */
.icon-comment:before { content: '\e848'; } /* '' */
.icon-chat:before { content: '\e849'; } /* '' */
.icon-chat-empty:before { content: '\e84a'; } /* '' */
.icon-bell:before { content: '\e84b'; } /* '' */
.icon-bell-alt:before { content: '\e84c'; } /* '' */
.icon-attention-alt:before { content: '\e84d'; } /* '' */
.icon-print:before { content: '\e84e'; } /* '' */
.icon-edit:before { content: '\e84f'; } /* '' */
.icon-forward:before { content: '\e850'; } /* '' */
.icon-reply:before { content: '\e851'; } /* '' */
.icon-reply-all:before { content: '\e852'; } /* '' */
.icon-eye:before { content: '\e853'; } /* '' */
.icon-tag:before { content: '\e854'; } /* '' */
.icon-tags:before { content: '\e855'; } /* '' */
.icon-lock-open-alt:before { content: '\e856'; } /* '' */
.icon-lock-open:before { content: '\e857'; } /* '' */
.icon-lock:before { content: '\e858'; } /* '' */
.icon-home:before { content: '\e859'; } /* '' */
.icon-info:before { content: '\e85a'; } /* '' */
.icon-help:before { content: '\e85b'; } /* '' */
.icon-search:before { content: '\e85c'; } /* '' */
.icon-flapping:before { content: '\e85d'; } /* '' */
.icon-rewind:before { content: '\e85e'; } /* '' */
.icon-chart-line:before { content: '\e85f'; } /* '' */
.icon-bell-off:before { content: '\e860'; } /* '' */
.icon-bell-off-empty:before { content: '\e861'; } /* '' */
.icon-plug:before { content: '\e862'; } /* '' */
.icon-eye-off:before { content: '\e863'; } /* '' */
.icon-arrows-cw:before { content: '\e864'; } /* '' */
.icon-cw:before { content: '\e865'; } /* '' */
.icon-host:before { content: '\e866'; } /* '' */
.icon-thumbs-up:before { content: '\e867'; } /* '' */
.icon-thumbs-down:before { content: '\e868'; } /* '' */
.icon-spinner:before { content: '\e869'; } /* '' */
.icon-attach:before { content: '\e86a'; } /* '' */
.icon-keyboard:before { content: '\e86b'; } /* '' */
.icon-menu:before { content: '\e86c'; } /* '' */
.icon-wifi:before { content: '\e86d'; } /* '' */
.icon-moon:before { content: '\e86e'; } /* '' */
.icon-chart-pie:before { content: '\e86f'; } /* '' */
.icon-chart-area:before { content: '\e870'; } /* '' */
.icon-chart-bar:before { content: '\e871'; } /* '' */
.icon-beaker:before { content: '\e872'; } /* '' */
.icon-magic:before { content: '\e873'; } /* '' */
.icon-spin6:before { content: '\e874'; } /* '' */
.icon-down-small:before { content: '\e875'; } /* '' */
.icon-left-small:before { content: '\e876'; } /* '' */
.icon-right-small:before { content: '\e877'; } /* '' */
.icon-up-small:before { content: '\e878'; } /* '' */
.icon-pin:before { content: '\e879'; } /* '' */
.icon-angle-double-left:before { content: '\e87a'; } /* '' */
.icon-angle-double-right:before { content: '\e87b'; } /* '' */
.icon-circle:before { content: '\e87c'; } /* '' */
.icon-info-circled:before { content: '\e87d'; } /* '' */
.icon-twitter:before { content: '\e87e'; } /* '' */
.icon-facebook-squared:before { content: '\e87f'; } /* '' */
.icon-gplus-squared:before { content: '\e880'; } /* '' */
.icon-attention-circled:before { content: '\e881'; } /* '' */
.icon-check:before { content: '\e883'; } /* '' */
.icon-reschedule:before { content: '\e884'; } /* '' */
.icon-warning-empty:before { content: '\e885'; } /* '' */
.icon-th-list:before { content: '\f009'; } /* '' */
.icon-th-thumb-empty:before { content: '\f00b'; } /* '' */
.icon-github-circled:before { content: '\f09b'; } /* '' */
.icon-angle-double-up:before { content: '\f102'; } /* '' */
.icon-angle-double-down:before { content: '\f103'; } /* '' */
.icon-angle-left:before { content: '\f104'; } /* '' */
.icon-angle-right:before { content: '\f105'; } /* '' */
.icon-angle-up:before { content: '\f106'; } /* '' */
.icon-angle-down:before { content: '\f107'; } /* '' */
.icon-history:before { content: '\f1da'; } /* '' */
.icon-binoculars:before { content: '\f1e5'; } /* '' */